"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const webpack_1 = tslib_1.__importDefault(require("webpack"));
const chalk_1 = tslib_1.__importDefault(require("chalk"));
const path_1 = tslib_1.__importDefault(require("path"));
const FileSizeReporter_1 = tslib_1.__importDefault(require("react-dev-utils/FileSizeReporter"));
const webpack_config_1 = require("../config/webpack-config");
const config_1 = require("../config/config");
const utils_1 = require("./utils");
const fs_1 = tslib_1.__importDefault(require("fs"));
process.env.NODE_ENV = 'production';
const config = config_1.getMergedConfig();
function runCompile(webpackConfig, isClient = false) {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        return new Promise(((resolve, reject) => {
            webpack_1.default(webpackConfig).run(((err, stats) => {
                if (err) {
                    console.log(chalk_1.default.redBright(err.message));
                    reject();
                    return;
                }
                const { warnings, errors } = stats.toJson();
                if (warnings.length) {
                    console.log(chalk_1.default.yellowBright('Compiled with warnings'));
                    warnings.forEach(warning => {
                        console.log(chalk_1.default.yellowBright(warning));
                    });
                }
                if (errors.length) {
                    console.log(chalk_1.default.redBright('Compiled with errors'));
                    errors.forEach(error => {
                        console.log(chalk_1.default.redBright(error));
                    });
                    reject();
                    return;
                }
                if (isClient) {
                    console.log(chalk_1.default.green('File sizes after gzip:\n'));
                    FileSizeReporter_1.default.printFileSizesAfterBuild(stats, {
                        root: path_1.default.join(config.distDir, 'client'),
                        sizes: {},
                    }, path_1.default.join(config.distDir, 'client'), 300);
                    console.log('\n');
                }
                resolve();
            }));
        }));
    });
}
function build() {
    var _a, _b;
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        try {
            console.log(chalk_1.default.green('Start compile...'));
            const clientConfig = webpack_config_1.createWebpackConfig(false);
            const serverConfig = webpack_config_1.createWebpackConfig(true);
            // clear output directory and copy static files
            const serverOutput = (_a = serverConfig.output) === null || _a === void 0 ? void 0 : _a.path;
            const clientOutput = (_b = clientConfig.output) === null || _b === void 0 ? void 0 : _b.path;
            utils_1.deleteDir(config.distDir);
            // create output directory
            fs_1.default.mkdirSync(config.distDir);
            const publicDir = path_1.default.join(config.rootDir, 'public');
            utils_1.copyDir(publicDir, serverOutput);
            utils_1.copyDir(publicDir, clientOutput);
            console.log(chalk_1.default.green('Start to build server bundle...'));
            yield runCompile(serverConfig);
            console.log(chalk_1.default.green('Start to build client bundle...'));
            yield runCompile(clientConfig, true);
            console.log(chalk_1.default.green('Compiled successfully.'));
        }
        catch (e) {
            process.exit(1);
        }
    });
}
build();
